import java.io.*;
import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.media.jai.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.util.*;
import java.beans.*;
import java.net.URL;

/*
   Copyright 2000 Joseph McMurry

   Released under the GPL (version 2)
    (see GPL.txt for more info)
*/

/**
 * @author Joseph McMurry <BR>
 * 12/23/2000             <BR>
 * JImageView application <BR>
 * Allows viewing of GIF, JPEG, PNG, TIF, and BMP (Win95 type) images <BR>
 * @version 0.2pre
 * @since JDK 1.2.2
 */
public class ImageViewer extends JFrame
                         implements ActionListener,
                                      MouseListener {

	private JDesktopPane desktopPane;
	private ImageInternalFrame frame;
	private JFileChooser fileChooser;
	private FileDialog fileDialog;
	private JComboBox scaleChoice;
	private PrefsDialog prefsDialog;
	private ScaleDialog scaleDialog;
	private JPopupMenu popup;
	private JToolBar toolBar;
	private JPanel contentPane;

	private boolean fileChooserCreated;
	private boolean prefsDialogCreated;
	private boolean scaleDialogCreated;
	private boolean useSwingFileChooser;
	private boolean previewImages;
	private boolean useJavaLF;
	private boolean useFileFilters;
	private boolean useFileTypeIcons;

	private OpenFrameList openFrameList;
	private int numFramesCreated; // the number of internal frames created so far
	private int openFrameCount; // the number of open internal frames
	private String[] scaleSizes;
        private Properties properties;


        // constants
        static final String configFilename = ".jimageviewrc";

        static final String xS = "x";
        static final String slashS = "/";
        static final String oneS = "1";

        static final String openS = "open";
        static final String closeS = "close";
        static final String exitS = "exit";
        static final String scaleS = "scale";
        static final String fullScreenS = "fullScreen";
        static final String aboutS = "about";
        static final String scaleChoiceS = "scaleChoice";
        static final String preferencesS = "preferences";
        static final String zoomInS = "zoomIn";
        static final String zoomOutS = "zoomOut";

        static final String trueS = "true";
        static final String falseS = "false";

        static final String userHomeS = "user.home";

	static final int xOffset = 30;
	static final int yOffset = 30;
	static final int minWidth = 100;
	static final int minHeight = 100;
	static final int maxWidth = 700;
	static final int maxHeight = 500;


	public ImageViewer(String[] args) {

		initBaseVars();

		openFrameList = new OpenFrameList();

		loadProperties();

		setLF();

                createMenus();
                createPopupMenu();

                contentPane = new JPanel();
                contentPane.setLayout( new BorderLayout() );
                createToolBar();
                String toolBarPos = properties.getProperty(PropertiesConstants.toolBarPosS);
                String toolBarLayout;
                if ( toolBarPos.equals(PropertiesConstants.topS) )
                   toolBarLayout = BorderLayout.NORTH;
                else if ( toolBarPos.equals(PropertiesConstants.bottomS) )
                   toolBarLayout = BorderLayout.SOUTH;
                else if ( toolBarPos.equals(PropertiesConstants.leftS) )
                   toolBarLayout = BorderLayout.WEST;
                else // if (right)
                   toolBarLayout = BorderLayout.EAST;
                contentPane.add( toolBar, toolBarLayout );
                desktopPane = new JDesktopPane();
		contentPane.add( desktopPane, BorderLayout.CENTER );

		setContentPane(contentPane);

                String prop = properties.getProperty(PropertiesConstants.dragModeS);
                if ( !(prop).equals(PropertiesConstants.liveS) ) { // if dragMode != live
                   // make dragging internal frames faster
                   desktopPane.putClientProperty("JDesktopPane.dragMode", PropertiesConstants.outlineS);
                }

		openMainWindow();

                // open the file named on the command line, if any
                if ( args.length > 0 )
                   showImageAI( new File(args[0]) );

	}


	private void initBaseVars() {

		fileChooserCreated = false;
		prefsDialogCreated = false;
		scaleDialogCreated = false;

		numFramesCreated = 0;
		openFrameCount = 0;

	}

	private void setLF() {

		try {
		     if (useJavaLF) 
                        UIManager.setLookAndFeel( UIManager.getCrossPlatformLookAndFeelClassName() );
                     else {
		        String os = System.getProperty("os.name"); 
			int index = os.indexOf("Linux");
			if (index > -1) // use Motif L&F on Linux
		           UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
                        else
			   UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
		     }	
                }
                catch (Exception e) {}

	}

	private void openMainWindow() {

		setTitle("JImageView");

	        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

	        String prop = properties.getProperty(PropertiesConstants.fullScreenS);
	        boolean fullScreen = Boolean.valueOf(prop).booleanValue();
		int width, height;

	        if (!fullScreen) {
                   width = Integer.parseInt( properties.getProperty(PropertiesConstants.widthS) );
	           height = Integer.parseInt( properties.getProperty(PropertiesConstants.heightS) );
	        }
	        else { // fullScreen
	           width = (int)d.getWidth();
	           height = (int)d.getHeight();
	        }
	        pack();
	        setSize(width, height);

	        // center the imageViewer window (JFrame) on the screen
	        Rectangle rect = new Rectangle(new Point(0,0), d);
                setLocation( (rect.x + ((rect.width - width)/2)),
		               (rect.y + ((rect.height - height)/2)) );

		setVisible(true);

	}

        /**
         * Reads the configuration file
         * into the Properties object.
         */
	private void loadProperties() {

     		properties = new Properties();
     		FileInputStream in = null;
     		BufferedInputStream bis = null;
     		String dir = System.getProperty(userHomeS);
     		String configFileWPath = dir+slashS+configFilename;
     		try {
     		     in = new FileInputStream(configFileWPath);
                     bis = new BufferedInputStream(in);
                     properties.load(bis);
                     String prop = properties.getProperty(PropertiesConstants.useSwingFileChooserS);
	             useSwingFileChooser = Boolean.valueOf(prop).booleanValue();
	             prop = properties.getProperty(PropertiesConstants.previewImagesS);
	             previewImages = Boolean.valueOf(prop).booleanValue();
	             prop = properties.getProperty(PropertiesConstants.useJavaLFS);
	             useJavaLF = Boolean.valueOf(prop).booleanValue();
	             prop = properties.getProperty(PropertiesConstants.useFileFiltersS);
	             useFileFilters = Boolean.valueOf(prop).booleanValue();
	             prop = properties.getProperty(PropertiesConstants.useFileTypeIconsS);
	             useFileTypeIcons = Boolean.valueOf(prop).booleanValue();
	        }
                catch(Exception e) {
                     // assume defaults for config file since one does not exit
                     System.out.println("Configuration file does not exist");
                     useSwingFileChooser = true;
                     previewImages = true;
                     useJavaLF = true;
                     useFileFilters = true;
                     useFileTypeIcons = true;
                     properties.setProperty(PropertiesConstants.useSwingFileChooserS, trueS);
                     properties.setProperty(PropertiesConstants.dragModeS, PropertiesConstants.outlineS);
                     properties.setProperty( PropertiesConstants.imageDirS, System.getProperty(userHomeS) );
                     properties.setProperty(PropertiesConstants.widthS, "730");
                     properties.setProperty(PropertiesConstants.heightS, "530");
                     properties.setProperty(PropertiesConstants.fullScreenS, falseS);
                     properties.setProperty(PropertiesConstants.toolBarPosS, PropertiesConstants.topS);
                     properties.setProperty(PropertiesConstants.previewImagesS, trueS);
                     properties.setProperty(PropertiesConstants.useJavaLFS, trueS);
                     properties.setProperty(PropertiesConstants.useFileFiltersS, trueS);
                     properties.setProperty(PropertiesConstants.useFileTypeIconsS, trueS);

                     // write default config file to disk
                     try {
                          FileOutputStream out = new FileOutputStream(configFileWPath);
             		  BufferedOutputStream bos = new BufferedOutputStream(out);
                          properties.store(bos, null);
                          bos.close();
                          out.close();
                     }
                     catch (IOException ioe) {
                     	  System.err.println("Properties file could not be saved");
                     }
                }

        }

	private void createMenus() {

		JMenuBar menuBar = new JMenuBar();

                // create "File" menu
                JMenu fileMenu = new JMenu("File");
                fileMenu.setMnemonic(KeyEvent.VK_F);
                JMenuItem open, close, quit;
                open = new JMenuItem("Open...");
                open.setMnemonic(KeyEvent.VK_O);
                open.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
                open.addActionListener(this);
                open.setActionCommand(openS);
                close = new JMenuItem("Close");
                close.setMnemonic(KeyEvent.VK_C);
                close.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
                close.addActionListener(this);
                close.setActionCommand(closeS);
                quit = new JMenuItem("Exit");
                quit.setMnemonic(KeyEvent.VK_X);
                quit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
                quit.addActionListener(this);
                quit.setActionCommand(exitS);
                fileMenu.add(open);
                fileMenu.add(close);
                fileMenu.addSeparator();
                fileMenu.add(quit);
                menuBar.add(fileMenu);

                // create "Edit" Menu
                JMenu editMenu = new JMenu("Edit");
                editMenu.setMnemonic(KeyEvent.VK_E);
                JMenuItem prefs = new JMenuItem("Preferences...");
                prefs.setMnemonic(KeyEvent.VK_P);
                prefs.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
                prefs.addActionListener(this);
                prefs.setActionCommand(preferencesS);
                editMenu.add(prefs);
                menuBar.add(editMenu);

                // create "View" Menu
                JMenu viewMenu = new JMenu("View");
                viewMenu.setMnemonic(KeyEvent.VK_V);
                JMenuItem scale, fullScreen;
                scale = new JMenuItem("Scale...");
                scale.setMnemonic(KeyEvent.VK_S);
                scale.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
                scale.addActionListener(this);
                scale.setActionCommand(scaleS);
                viewMenu.add(scale);
                JMenuItem zoomIn, zoomOut;
                zoomIn = new JMenuItem("Zoom In");
                zoomIn.setMnemonic(KeyEvent.VK_I);
                zoomIn.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK));
                zoomIn.addActionListener(this);
                zoomIn.setActionCommand(zoomInS);
                viewMenu.add(zoomIn);
                zoomOut = new JMenuItem("Zoom Out");
                zoomOut.setMnemonic(KeyEvent.VK_O);
                zoomOut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
                zoomOut.addActionListener(this);
                zoomOut.setActionCommand(zoomOutS);
                viewMenu.add(zoomOut);
                viewMenu.addSeparator();
                fullScreen = new JMenuItem("Full Screen");
                fullScreen.setMnemonic(KeyEvent.VK_U);
                fullScreen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
                fullScreen.addActionListener(this);
                fullScreen.setActionCommand(fullScreenS);
                viewMenu.add(fullScreen);
                menuBar.add(viewMenu);

                // create "Help" Menu
                JMenu helpMenu = new JMenu("Help");
                helpMenu.setMnemonic(KeyEvent.VK_H);
                JMenuItem about = new JMenuItem("About");
                about.setMnemonic(KeyEvent.VK_A);
                about.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
                about.addActionListener(this);
                about.setActionCommand(aboutS);
                helpMenu.add(about);
                menuBar.add( Box.createHorizontalGlue() ); // space help menu to far right
                menuBar.add(helpMenu);

                setJMenuBar(menuBar);

	}

	/**
	 * Create the popup (right-click) menu.
	 */
        private void createPopupMenu() {

                popup = new JPopupMenu();

                JMenuItem scale, zoomIn, zoomOut, fullScreen;
                scale = new JMenuItem("Scale...");
                scale.setMnemonic(KeyEvent.VK_S);
                scale.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
                scale.addActionListener(this);
                scale.setActionCommand(scaleS);
                popup.add(scale);
                zoomIn = new JMenuItem("Zoom In");
                zoomIn.setMnemonic(KeyEvent.VK_I);
                zoomIn.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK));
                zoomIn.addActionListener(this);
                zoomIn.setActionCommand(zoomInS);
                popup.add(zoomIn);
                zoomOut = new JMenuItem("Zoom Out");
                zoomOut.setMnemonic(KeyEvent.VK_O);
                zoomOut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
                zoomOut.addActionListener(this);
                zoomOut.setActionCommand(zoomOutS);
                popup.add(zoomOut);
                popup.addSeparator();
                fullScreen = new JMenuItem("Full Screen");
                fullScreen.setMnemonic(KeyEvent.VK_U);
                fullScreen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
                fullScreen.addActionListener(this);
                fullScreen.setActionCommand(fullScreenS);
                popup.add(fullScreen);

        }

	private void createToolBar() {

		toolBar = new JToolBar();

		URL iconURL = ClassLoader.getSystemResource("icons/Open24.gif");
		ImageIcon open = new ImageIcon(iconURL);
	        JButton openButton = new JButton(open);
      	 	openButton.setToolTipText("Open");
	 	openButton.addActionListener(this);
	 	openButton.setActionCommand(openS);
	 	toolBar.add(openButton);

		toolBar.addSeparator();

	 	// create combo box for scaling an image
	 	scaleSizes = new String[7];
	 	int i;
	 	int j = 0;
	 	for (i = 4; i > 0; i--) { // create items 4x to 1x
	 	    scaleSizes[j] = ( new Integer(i) ).toString() + xS;
	 	    j++;
	 	}
	 	for (i = 2; i < 5; i++) { // create items 1/2x to 1/4x
	 	    scaleSizes[j] = oneS + slashS + ( new Integer(i) ).toString() +xS;
	 	    j++;
	 	}
                scaleChoice = new JComboBox(scaleSizes);
                scaleChoice.setEditable(true);
                scaleChoice.setSelectedIndex(3); // set 1x to be the selected value
                scaleChoice.addActionListener(this);
                scaleChoice.setActionCommand(scaleChoiceS);
                scaleChoice.setToolTipText("Scale/Zoom");
                toolBar.add(scaleChoice);

	}

	public void actionPerformed(ActionEvent e) {

		String actionCommand = e.getActionCommand();

		if ( actionCommand.equals(openS) )
		   openAction();
		else if ( actionCommand.equals(closeS) )
		   closeAction();
	        else if ( actionCommand.equals(exitS) )
		   exitAction();
		else if ( actionCommand.equals(preferencesS) )
		   prefsAction();
		else if ( actionCommand.equals(scaleS) ) // scale menu item
		   scaleAction();
		else if ( actionCommand.equals(fullScreenS) )
		   fullScreenAction();
	        else if ( actionCommand.equals(aboutS) )
                   aboutAction();
                else if ( actionCommand.equals(scaleChoiceS) ) // scale combo box
                   scaleChoiceAction();
                else if ( actionCommand.equals(zoomInS) )
                   zoomAction(-1);
                else if ( actionCommand.equals(zoomOutS) )
                   zoomAction(1);

	}

	public void resetDragMode(String dragMode) {

		desktopPane.putClientProperty("JDesktopPane.dragMode", dragMode);

	}

	public void resetFileChooser() {

		fileChooserCreated = false;
		String prop = properties.getProperty(PropertiesConstants.useSwingFileChooserS);
	        useSwingFileChooser = Boolean.valueOf(prop).booleanValue();
	        prop = properties.getProperty(PropertiesConstants.previewImagesS);
	        previewImages = Boolean.valueOf(prop).booleanValue();
	        prop = properties.getProperty(PropertiesConstants.useJavaLFS);
	        useJavaLF = Boolean.valueOf(prop).booleanValue();
	        prop = properties.getProperty(PropertiesConstants.useFileFiltersS);
	        useFileFilters = Boolean.valueOf(prop).booleanValue();
	        prop = properties.getProperty(PropertiesConstants.useFileTypeIconsS);
	        useFileTypeIcons = Boolean.valueOf(prop).booleanValue();

	}

	/**
	 * Opens an existing image file.
	 */
	private void openAction() {

		File file;

		if (useSwingFileChooser) {

		   if (!fileChooserCreated) {
		      fileChooser = new JFileChooser();
		      fileChooser.setMultiSelectionEnabled(true);
		      String prop = properties.getProperty(PropertiesConstants.imageDirS);
		      fileChooser.setCurrentDirectory( new File( prop ) );
		      if (useFileFilters)
		         fileChooser.addChoosableFileFilter( new ImageFilter() );
                      if (useFileTypeIcons)
                         fileChooser.setFileView( new ImageFileView() );
                      if (previewImages)
                         fileChooser.setAccessory( new ImagePreview(fileChooser) );
                      fileChooserCreated = true;
                   }

		   int returnVal = fileChooser.showOpenDialog(this);

                   if (returnVal == JFileChooser.APPROVE_OPTION) {
                      //file = fileChooser.getSelectedFile(); // only can open one file at a time
                      File[] files = getSelectedFiles(fileChooser);
                      for (int i = 0; i < files.length; i++)
                          showImageAI(files[i]); // use JAI for all images to support scaling
                   }

                }
                else { // native file dialog

		   if (!fileChooserCreated) {
                      fileDialog = new FileDialog(this,"Open",FileDialog.LOAD);
                      fileDialog.setDirectory( properties.getProperty(PropertiesConstants.imageDirS) );
                      fileChooserCreated = true;
                   }

		   fileDialog.setFile(Utils.allTypes); // file filter for Win32		   
                   
                   fileDialog.setVisible(true);
                   String filename;
                   if ( (filename = fileDialog.getFile() ) != null) {
                      file = new File( fileDialog.getDirectory() + filename );
                      showImageAI(file);
		   }
                }

	}
	
	/**
	 * This is an ugly hack to allow getting multiple files
	 * from a JFileChooser at once.
	 */
	public static File[] getSelectedFiles(JFileChooser chooser) {

        	Container c1 = (Container)chooser.getComponent(3);
		JList list = null;

		while (c1 != null) {
                      Container c = (Container)c1.getComponent(0);
                      if (c instanceof JList) {
                         list = (JList)c;
                         break;
                      }
                      c1 = c;
                }

		Object[] entries = list.getSelectedValues();
                File[] files = new File[entries.length];

		for (int k=0; k<entries.length; k++) {
                    if (entries[k] instanceof File)
                       files[k] = (File)entries[k];
                }

                return files;

        }

	/**
	 * Closes and disposes of the currently selected internal frame.
	 */
	private void closeAction() {

		ImageInternalFrame frameToClose;

		frameToClose = openFrameList.getSelectedFrame();
		if (frameToClose != null) {
		   openFrameCount--;
		   frameToClose.setVisible(false);
		   frameToClose.dispose();
		   frameToClose = null;
		}

	}

	private void exitAction() {

		System.exit(0);

	}

	private void prefsAction() {

		if (!prefsDialogCreated) {
                   prefsDialog = new PrefsDialog(this, properties);
                   prefsDialogCreated = true;
		}

		prefsDialog.setVisible(true);

	}

	private void scaleAction() {

		if (!scaleDialogCreated) {
		   scaleDialog = new ScaleDialog(this, scaleSizes);
		   scaleDialogCreated = true;
		}

		scaleDialog.setVisible(true);

	}

	private void fullScreenAction() {

		final JWindow window = new JWindow();
		final Container contentPane;

		final ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();

		if (selectedFrame != null) {
		   contentPane = selectedFrame.getContentPane();
		   window.setContentPane(contentPane);
		   Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		   window.setSize( (int)d.getWidth(), (int)d.getHeight() );
		   window.addMouseListener(
		   	new MouseAdapter() {
		   		public void mousePressed(MouseEvent e) {
                                             window.setVisible(false);
                                             selectedFrame.setContentPane(contentPane);
                                             selectedFrame.revalidate();
                                             window.dispose();
                                }
                        }
                   );
                   window.addKeyListener(
                   	new KeyAdapter() {
                        	public void keyPressed(KeyEvent e) {
                                	int keyCode =  e.getKeyCode();
                                        if (keyCode == KeyEvent.VK_ESCAPE)
                                           window.setVisible(false);
                                           selectedFrame.setContentPane(contentPane);
                                           selectedFrame.revalidate();
                                           window.dispose();
                                }
                   	}
                   );
		   window.setVisible(true);
		   window.requestFocus();
		}

	}

        /**
         * Opens the Help->About dialog.
         */
        private void aboutAction() {

        	JDialog aboutDialog = new JDialog(this, "About");

        	String contents = "       JImageView\n";
        	contents += "       Written by Joseph McMurry (jcmcmurr@eos.ncsu.edu)\n";
        	contents += "       Distributed under the terms of the GPL (version 2)\n\n";

        	try {
        	     StringBuffer all;
        	     URL url = ClassLoader.getSystemResource("GPL.txt");
        	     BufferedReader br = new BufferedReader( new InputStreamReader( url.openStream() ) );

                     String part;
	             all = new StringBuffer("");

	             while ( ( part = br.readLine() ) != null) {
	                   all.append(part);
	                   all.append('\n');
                     }

	             br.close();

	             contents += all.toString();
                }
                catch (Exception e) { }

        	JTextArea textArea = new JTextArea(30, 40);
        	textArea.setEditable(false);
        	JScrollPane scrollPane = new JScrollPane(textArea);
                textArea.setText(contents);
                textArea.setCaretPosition(0);

                aboutDialog.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
                aboutDialog.setContentPane(scrollPane);
                aboutDialog.pack();

                int width = 600;
                int height = 570;
                aboutDialog.setSize(width, height);

                // center the dialog on the screen
                Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
                Rectangle rect = new Rectangle(new Point(0,0), d);
                aboutDialog.setLocation( (rect.x + ((rect.width - width)/2)),
		               (rect.y + ((rect.height - height)/2)) );
                aboutDialog.setVisible(true);

        }

        private void scaleChoiceAction() {

        	ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();

		if (selectedFrame != null) {

                   // get the selected item in the scaleChoice (combo box)
                   String scaleString = (String)scaleChoice.getSelectedItem();

                   scale(scaleString, selectedFrame);

                }

	}

	/**
	 * Zoom in or out on the selected frame. <BR>
	 * inOut == -1 if zooming in, 1 if zooming out
	 */
	private void zoomAction(int inOut) {

        	ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();

		String scaleString;

		if (selectedFrame != null) {

                   scaleString = selectedFrame.getScaleFactor();
                   if ( scaleString.endsWith(xS) ) {
		      // remove the "x" from the end
                      scaleString = scaleString.substring(0, scaleString.length() - 1);
                   }

                   // check to see if the selected item is a fraction
                   int index = scaleString.indexOf('/');

                   float scale;
                   try {

                        if (index == -1) { // integer
                           scale = Float.parseFloat(scaleString);
                        }
                        else { // fraction (ie 1/2)
                           float n, d;
                           n = Float.parseFloat( scaleString.substring(0,1) ); // the numerator
                           d = Float.parseFloat( scaleString.substring(2) ); // the denominator
                           scale = n/d;
                        }
                        if (inOut == -1)
                           scale *= 2F; // zoom in (2x)
                        else
                           scale *= .5F; // zoom out (.5x)

			scaleString = Float.toString(scale) + xS;

                        scaleImage(scale, selectedFrame);
		   }
		   catch (NumberFormatException nfe) {}

		   selectedFrame.setScaleFactor(scaleString);
                   scaleChoice.getEditor().setItem(scaleString);

                }

	}

	public ImageInternalFrame getSelectedFrame() {

		return openFrameList.getSelectedFrame();

	}

	public void scale(String scaleString, ImageInternalFrame selectedFrame) {

		String backupSS = scaleString;

		if ( scaleString.endsWith(xS) ) {
		   // remove the "x" from the end
                   scaleString = scaleString.substring(0, scaleString.length() - 1);
                }

                // check to see if the selected item is a fraction
                int index = scaleString.indexOf('/');

                float scale;
                try {

                    if (index == -1) { // integer
                       scale = Float.parseFloat(scaleString);
                    }
                    else { // fraction (ie 1/2)
                       float n, d;
                       n = Float.parseFloat( scaleString.substring(0,1) ); // the numerator
                       d = Float.parseFloat( scaleString.substring(2) ); // the denominator
                       scale = n/d;
                    }

		    scaleImage(scale, selectedFrame);

                }
                catch (NumberFormatException nfe) {
                      JOptionPane.showMessageDialog(this, "\""+scaleString+"\" is not a valid scale factor.",
		                                     "Error", JOptionPane.ERROR_MESSAGE);
                }

                selectedFrame.setScaleFactor(backupSS);
                scaleChoice.getEditor().setItem(backupSS);

	}

	/**
	 * Performs the actual image scaling.
	 */
	private void scaleImage(float scale, ImageInternalFrame selectedFrame) {

                BufferedImage bufferedImage = (BufferedImage)selectedFrame.getImage();

		// Create a standard bilinear interpolation object to be
                //  used with the "scale" operator.
                Interpolation interp = Interpolation.getInstance(Interpolation.INTERP_BILINEAR);

		// Stores the required input source and parameters in a
                //  ParameterBlock to be sent to the operation registry,
                //   and eventually to the "scale" operator.
                ParameterBlock pb = new ParameterBlock();
                pb.addSource(bufferedImage);
                pb.add(scale);
                pb.add(scale);
                pb.add(0.0F);
                pb.add(0.0F);
                pb.add(interp);

		// Create an operator to scale the image
                RenderedOp image2 = JAI.create(scaleS, pb);

		// convert image2 to a BufferedImage to improve performance
		bufferedImage = image2.getAsBufferedImage();

                showScaledImage(bufferedImage, selectedFrame);

        }

        private void showScaledImage(BufferedImage bufferedImage, ImageInternalFrame selectedFrame) {

		DisplayJAI imagePanel = new DisplayJAI(bufferedImage);
        	JScrollPane scrollPane = new JScrollPane(imagePanel);

		selectedFrame.setContentPane(scrollPane);

        }

	/**
	 * Opens a JInternalFrame with the image specified
	 * by the File "file".  Called for all images.
	 */
        private void showImageAI(File file) {

		// load the image
                PlanarImage planarImage = JAI.create("fileload", file.getAbsolutePath() );
                BufferedImage bufferedImage = planarImage.getAsBufferedImage();

        	// add the image to the scroll pane
        	DisplayJAI imagePanel = new DisplayJAI(bufferedImage);
        	JScrollPane scrollPane = new JScrollPane(imagePanel);

                addToFrame(scrollPane, file);
                frame.setImage(bufferedImage); // set the frame's image

                int width = bufferedImage.getWidth() + 40;
                int height = bufferedImage.getHeight() + 40;

                openFrame(width, height);

        }

        // empty mouse methods must be here so that this
        //  class meets the mouse listener interface

        public void mouseClicked(MouseEvent me) {}

        public void mouseEntered(MouseEvent me) {}

        public void mouseExited(MouseEvent me) {}

        public void mousePressed(MouseEvent me) {
	
		if (me.isPopupTrigger()) 
                   popup.show(me.getComponent(), me.getX(), me.getY());	
	
	}

        /**
         * Invoked when a mouse button is released - used
         * to listen for mouse clicks on the images,
         * so that the resize window can be opened.
         */
        public void mouseReleased(MouseEvent me) {
		        
		if (me.isPopupTrigger()) 
                   popup.show(me.getComponent(), me.getX(), me.getY());
	
        }

        /**
         * Increments openFrameCount, constructs the internal frame,
         * sets its content pane, packs it, and ands an internal
         * frame listener to it.
         */
        private synchronized void addToFrame(JScrollPane scrollPane, File file) {

		frame = new ImageInternalFrame( file.getName(),        // title
                                         	  true,                // resizable
                                                   true,               // closable
                                                    true,              // maximizable
                                                     true,             // iconifiable
                                                      numFramesCreated,
                                                       this );
                numFramesCreated++;
                openFrameList.addElement(frame);
                frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
                frame.setContentPane(scrollPane);
                frame.addMouseListener(this);
                frame.pack();
                openFrameCount++;

        }

        private void openFrame(int width, int height) {

                if ( (width < minWidth) || (height < minHeight) ) // below min height or width
                   frame.setSize( minWidth, minHeight );
                else if ( (width < maxWidth) && (height < maxHeight) )
                   frame.setSize( width, height );
                else if ( (width >= maxWidth) && (height >= maxHeight) ) // exceeds max width and height
                   frame.setSize( maxWidth, maxHeight );
                else if (width >= maxWidth) // exceeds max width
                   frame.setSize( maxWidth, height );
                else if (height >= maxHeight) // exceeds max height
                   frame.setSize( width, maxHeight );

                // tile the internal frames
                int xLoc, yLoc;
                xLoc = xOffset * ( openFrameCount % 10 );
                yLoc = yOffset * ( openFrameCount % 10 );
                frame.setLocation(xLoc, yLoc);
                frame.setVisible(true); //necessary as of kestrel
                desktopPane.add(frame);
                try {
                  frame.setSelected(true);
                }
                catch (java.beans.PropertyVetoException e) {}

        }

        public void decrementOpenFrameCount() {

        	openFrameCount--;

        }

        public void updateSelectedFrame(int index, String scaleFactor) {

        	openFrameList.setSelectedFrame(index);

        	// set the scale factor displayed in the combo box to scaleFactor
        	scaleChoice.getEditor().setItem(scaleFactor);

        }

	public static void main(String[] args) {

                ImageViewer imageViewer = new ImageViewer(args);
		imageViewer.addWindowListener(
			     new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
				   System.exit(0);
				}
		             }
		);

	}

}
