import java.io.*;
import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
//import javax.media.jai.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.util.*;
import java.beans.*;

// Released under the GPL (version 2)

/**
 * @author Joseph McMurry <BR>
 * 11/24/2000              <BR>
 * JImageView application <BR>
 * Allows viewing of GIF and JPEG images w/o JAI<BR>
 * @version 0.1.1
 * @since JDK 1.3
 */
public class ImageViewerNOAI extends JFrame implements ActionListener {

	private JDesktopPane desktopPane;
	private JInternalFrame frame;
	private JFileChooser fileChooser;
	private boolean fileChooserCreated;
	private int openFrameCount;

	static final int xOffset = 30;
	static final int yOffset = 30;
	static final int minWidth = 100;
	static final int minHeight = 100;
	static final int maxWidth = 700;
	static final int maxHeight = 500;

	public ImageViewerNOAI() {

		fileChooserCreated = false;
		openFrameCount = 0;

                createMenus();

                JPanel contentPane = new JPanel();
                contentPane.setLayout( new BorderLayout() );

		JToolBar toolBar = createToolBar();
		contentPane.add( toolBar, BorderLayout.NORTH );

		desktopPane = new JDesktopPane();
		contentPane.add( desktopPane, BorderLayout.CENTER );

		setContentPane(contentPane);

                // make dragging internal frames faster
                desktopPane.putClientProperty("JDesktopPane.dragMode", "outline");

	}

	private void createMenus() {

		JMenuBar menuBar = new JMenuBar();

                // create "File" menu
                JMenu fileMenu = new JMenu("File");
                JMenuItem open, close, quit;
                open = new JMenuItem("Open");
                open.addActionListener(this);
                open.setActionCommand("open");
                close = new JMenuItem("Close");
                close.addActionListener(this);
                close.setActionCommand("close");
                quit = new JMenuItem("Exit");
                quit.addActionListener(this);
                quit.setActionCommand("exit");
                fileMenu.add(open);
                fileMenu.add(close);
                fileMenu.add(quit);
                menuBar.add(fileMenu);

                // create "Edit" Menu
                JMenu editMenu = new JMenu("Edit");
                JMenuItem edit = new JMenuItem("Preferences...");
                editMenu.add(edit);
                menuBar.add(editMenu);

                // create "View" Menu
                JMenu viewMenu = new JMenu("View");
                JMenuItem fullScreen = new JMenuItem("Full Screen");
                fullScreen.addActionListener(this);
                fullScreen.setActionCommand("fullScreen");
                viewMenu.add(fullScreen);
                menuBar.add(viewMenu);

                // create "Help" Menu
                JMenu helpMenu = new JMenu("Help");
                JMenuItem about = new JMenuItem("About");
                about.addActionListener(this);
                about.setActionCommand("about");
                helpMenu.add(about);
                menuBar.add(Box.createHorizontalGlue()); // space help menu to far right
                menuBar.add(helpMenu);

                setJMenuBar(menuBar);

	}

	private JToolBar createToolBar() {

		JToolBar toolBar = new JToolBar();

		ImageIcon open = new ImageIcon("icons/Open24.gif");
	        JButton openButton = new JButton(open);
      	 	openButton.setToolTipText("Open");
	 	openButton.addActionListener(this);
	 	openButton.setActionCommand("open");
	 	toolBar.add(openButton);

	 	return toolBar;

	}

	public void actionPerformed(ActionEvent e) {

		String actionCommand = e.getActionCommand();

		if ( actionCommand.equals("open") )
		   openAction();
		else if ( actionCommand.equals("close") )
		   closeAction();
	        else if ( actionCommand.equals("exit") )
		   exitAction();
		else if ( actionCommand.equals("fullScreen") )
		   fullScreenAction();
	        else if ( actionCommand.equals("about") )
                   aboutAction();

	}

	/**
	 * Opens an existing image file.
	 */
	private void openAction() {

		File file;

		if (!fileChooserCreated) {
		   fileChooser = new JFileChooser();
		   fileChooser.addChoosableFileFilter(new ImageFilter());
                   fileChooser.setFileView(new ImageFileView());
                   fileChooser.setAccessory(new ImagePreview(fileChooser));
                   fileChooserCreated = true;
                }

                int returnVal = fileChooser.showOpenDialog(this);

                if (returnVal == JFileChooser.APPROVE_OPTION) {
                   file = fileChooser.getSelectedFile();
                   String filename = file.getName();
                   if ( filename.endsWith(".jpg") || filename.endsWith(".JPG") ||
                         filename.endsWith(".gif") || filename.endsWith(".GIF") )
                      showImage(file);
                   /*else
                      showImageAI(file);*/
                }

	}

	/**
	 * Closes and disposes of the currently selected internal frame.
	 */
	private void closeAction() {

		JInternalFrame frameToClose;

		frameToClose = desktopPane.getSelectedFrame();
		if (frameToClose != null) {
		   openFrameCount--;
		   frameToClose.setVisible(false);
		   frameToClose.dispose();
		   frameToClose = null;
		}

	}

	private void exitAction() {

		System.exit(0);

	}

	private void fullScreenAction() {

		final JWindow window = new JWindow();
		final Container contentPane;

		final JInternalFrame selectedFrame = desktopPane.getSelectedFrame();

		if (selectedFrame != null) {
		   contentPane = selectedFrame.getContentPane();
		   window.setContentPane(contentPane);
		   Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		   window.setSize( (int)d.getWidth(), (int)d.getHeight() );
		   window.addMouseListener(
		   	new MouseAdapter() {
		   		public void mousePressed(MouseEvent e) {
                                             window.setVisible(false);
                                             selectedFrame.setContentPane(contentPane);
                                             selectedFrame.revalidate();
                                             window.dispose();
                                }
                        }
                   );
		   window.setVisible(true);
		}

	}

        /**
         * Opens the Help->About dialog.
         */
        private void aboutAction() {

        	JDialog aboutDialog = new JDialog(this, "About");
        	JPanel contentPane = new JPanel();
        	contentPane.setLayout( new FlowLayout() );

        	String contents = "       JImageView\n";
        	contents += "       Written by Joseph McMurry (jcmcmurr@eos.ncsu.edu)\n";
        	contents += "       Distributed under the terms of the GPL (version 2)\n\n";

        	try {
        	     StringBuffer all;
        	     FileReader fr = new FileReader("GPL.txt");
                     BufferedReader br = new BufferedReader(fr);

                     String part;
	             all = new StringBuffer("");

	             while ( ( part = br.readLine() ) != null) {
	                   all.append(part);
	                   all.append('\n');
                     }

	             br.close();
	             fr.close();

	             contents += all.toString();
                }
                catch (Exception e) { }

        	JTextArea textArea = new JTextArea(30, 40);
        	textArea.setEditable(false);
        	JScrollPane scrollPane = new JScrollPane(textArea);
                textArea.setText(contents);
                textArea.setCaretPosition(0);
                contentPane.add(scrollPane);

                aboutDialog.setDefaultCloseOperation(DISPOSE_ON_CLOSE);

                aboutDialog.setContentPane(contentPane);
                aboutDialog.pack();
                aboutDialog.setSize(500, 570);
                aboutDialog.setVisible(true);

        }

	/**
	 * Opens a JInternalFrame with the image specified
	 * by the File "file".  Called for images that are not
	 * GIF or JPEG.
	 */
        /*private void showImageAI(File file) {

		// load the image
                PlanarImage planarImage = JAI.create("fileload", file.getAbsolutePath() );
                BufferedImage bufferedImage = planarImage.getAsBufferedImage();

        	// add the image to the scroll pane
        	DisplayJAI imagePanel = new DisplayJAI(bufferedImage);
        	JScrollPane scrollPane = new JScrollPane(imagePanel);

                addToFrame(scrollPane, file);

                int width = bufferedImage.getWidth() + 40;
                int height = bufferedImage.getHeight() + 40;

                openFrame(width, height);

        }*/


	/**
	 * Opens a JInternalFrame with the image specified
	 * by the File "file".  Works only with GIF and JPEG images.
	 */
	private void showImage(File file) {

		ImageIcon imageIcon = createImageFrame(file);

                int width = imageIcon.getIconWidth() + 40;
                int height = imageIcon.getIconHeight() + 40;

		openFrame(width, height);

        }

        private ImageIcon createImageFrame(File file) {

                ImageIcon imageIcon;
                imageIcon = new ImageIcon( file.getAbsolutePath() );
		JScrollPane scrollPane = new JScrollPane();
		JLabel label = new JLabel(imageIcon);

		JPanel inner = new JPanel();
		inner.setLayout( new BorderLayout() );
		inner.add(label, BorderLayout.CENTER);
                scrollPane.getViewport().add(inner);

                addToFrame(scrollPane, file);

		return imageIcon;

        }

        /**
         * Increments openFrameCount, constructs the internal frame,
         * sets its content pane, packs it, and ands an internal
         * frame listener to it.
         */
        private void addToFrame(JScrollPane scrollPane, File file) {

		frame = new JInternalFrame( file.getName(), // title
                                     			      true,          // resizable
                                              		       true,         // closable
                                               			true,        // maximizable
                                                		 true);      // iconifiable
                frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        	frame.setContentPane(scrollPane);
                frame.pack();
                openFrameCount++;

        }

        private void openFrame(int width, int height) {

                if ( (width < minWidth) || (height < minHeight) ) // below min height or width
                   frame.setSize( minWidth, minHeight );
                else if ( (width < maxWidth) && (height < maxHeight) )
                   frame.setSize( width, height );
                else if ( (width >= maxWidth) && (height >= maxHeight) ) // exceeds max width and height
                   frame.setSize( maxWidth, maxHeight );
                else if (width >= maxWidth) // exceeds max width
                   frame.setSize( maxWidth, height );
                else if (height >= maxHeight) // exceeds max height
                   frame.setSize( width, maxHeight );

                // tile the internal frames
                int xLoc, yLoc;
                xLoc = xOffset * ( openFrameCount % 10 );
                yLoc = yOffset * ( openFrameCount % 10 );
                frame.setLocation(xLoc, yLoc);
                frame.setVisible(true); //necessary as of kestrel
                desktopPane.add(frame);
                try {
                  frame.setSelected(true);
                }
                catch (java.beans.PropertyVetoException e) {}

        }

	public static void main(String[] args) {

                int width, height;
		ImageViewerNOAI imageViewer = new ImageViewerNOAI();
		imageViewer.addWindowListener(
			     new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
				   System.exit(0);
				}
		             }
		);
		imageViewer.setTitle("JImageView");
	        width = 730;
	        height = 530;
	        imageViewer.pack();
	        imageViewer.setSize(width, height);

	        // center the imageViewer window (JFrame) on the screen
	        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
	        Rectangle rect = new Rectangle(new Point(0,0), d);
                imageViewer.setLocation( (rect.x + ((rect.width - width)/2)),
		                          (rect.y + ((rect.height - height)/2)) );

		imageViewer.setVisible(true);

	}

}
